﻿using System;
using FCS.Common;
using FCS.Interface;
using FCS.Models;
using FCS.Models.Entity;
using HtmlAgilityPack;
using Quartz;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using FCS.Models.DTO;

namespace FCS.Crawler.Basketball
{
    /// <summary>
    /// 篮球大小盘路
    /// </summary>
    public class B_SizeBallRankingJob : CommonJob, IJob
    {
        private static List<B_SizeBallRanking> SizeBallRankingList = new List<B_SizeBallRanking>();
        private static List<DataItemDetail> DateItem = new List<DataItemDetail>();
        private static List<B_Team> TeamList = new List<B_Team>();

        public B_SizeBallRankingJob()
        {
        }

        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
        }


        #region 更新数据
        /// <summary>
        /// 获取所有数据
        /// </summary>
        public void GetAll()
        {
            GetSqlString("");
        }

        /// <summary>
        /// 更新数据
        /// </summary>
        /// <param name="Year"></param>
        public void Click(string Year = "")
        {
            if (Year == "")
            {
                Year = DateTime.Now.Year.ToString();
            }
            GetSqlString(Year);
        }
        #endregion


        /// <summary>
        /// 解析数据
        /// </summary>
        /// <param name="sqlstring"></param>
        private void GetSqlString(string Year)
        {
            ThreadPool.SetMinThreads(10, 10);
            ThreadPool.SetMaxThreads(500, 500);
            TeamList = services.Query<B_Team>().ToList();
            SizeBallRankingList = services.Query<B_SizeBallRanking>().ToList();
            var dataItem = services.GetDataItem(DataItemDetailEnum.FootBallSizeBallType);
            var ds = services.Query<B_Grouping>("and Remark is not null", "", "distinct EventId,Season,Remark").ToList();
            if (Year != "")
            {
                ds = services.Query<B_Grouping>("and Remark is not null  and Season like '%" + Year + "%'", "", "distinct EventId,Season,Remark").ToList();
            }
            Task.Run(() =>
            {
                foreach (var item in ds)
                {
                    Task.Run(() =>
                    {
                        var eventId = item.EventId;
                        var season = item.Season;
                        var url = item.Remark.ToString().Replace("saiAll.do", "1/bigSmallBall.do");
                        GetData(url, eventId, season, dataItem.Where(o => o.ItemValue == "1").First().Id.ToString());
                        url = item.Remark.ToString().Replace("saiAll.do", "2/bigSmallBall.do");
                        GetData(url, eventId, season, dataItem.Where(o => o.ItemValue == "2").First().Id.ToString());
                        url = item.Remark.ToString().Replace("saiAll.do", "3/bigSmallBall.do");
                        GetData(url, eventId, season, dataItem.Where(o => o.ItemValue == "3").First().Id.ToString());
                    });
                }
            });
            int maxWorkerThreads, workerThreads;
            int maxportThreads, portThreads;

            while (true)
            {
                /*
                 GetAvailableThreads()：检索由 GetMaxThreads 返回的线程池线程的最大数目和当前活动数目之间的差值。
                 而GetMaxThreads 检索可以同时处于活动状态的线程池请求的数目。
                 通过最大数目减可用数目就可以得到当前活动线程的数目，如果为零，那就说明没有活动线程，说明所有线程运行完毕。
                 */
                ThreadPool.GetMaxThreads(out maxWorkerThreads, out maxportThreads);
                ThreadPool.GetAvailableThreads(out workerThreads, out portThreads);
                Thread.Sleep(1000);

                Trace.WriteLine("正在执行任务的线程数" + (maxWorkerThreads - workerThreads));
                if (maxWorkerThreads - workerThreads == 0)
                {
                    Console.WriteLine("Thread Finished!");
                    break;
                }
            }
            try
            {
                if (Year != "")
                {
                    services.Delete<B_SizeBallRanking>(" and Season like '" + Year + "%'");
                }
                services.SqlBulkCopyAdd<B_SizeBallRanking>(SizeBallRankingList);
                Trace.WriteLine("B_SizeBallRanking更新完毕");
            }
            catch (Exception ex)
            {

                throw;
            }

        }

        /// <summary>
        /// 解析排行榜数据
        /// </summary>
        /// <param name="url"></param>
        /// <param name="eventId"></param>
        /// <param name="season"></param>
        private void GetData(string url, string eventId, string season, string sizeBallType)
        {
            HtmlParameterDTO dtomodel = new HtmlParameterDTO();
            dtomodel.Url = url;
            HtmlDocument doc = CommonHelper.GetHtmlHtmlDocument(dtomodel);
            if (doc.DocumentNode.InnerHtml == "Termination" || doc.DocumentNode.InnerHtml == "" || doc.DocumentNode.SelectNodes("//*[@class='paiming']") == null)
            {
                return;
            }
            var team = doc.DocumentNode.SelectNodes("//*[@class='paiming']").First();

            HtmlDocument itemdoc = new HtmlDocument();
            itemdoc.LoadHtml(team.InnerHtml);
            var trList = itemdoc.DocumentNode.SelectNodes("//table/tr").ToList();
            trList.ForEach(p =>
            {
                if (!p.InnerHtml.Contains("<th"))
                {
                    HtmlDocument trdoc = new HtmlDocument();
                    trdoc.LoadHtml(p.InnerHtml);
                    var tdList = trdoc.DocumentNode.InnerHtml.Contains("td") ? trdoc.DocumentNode.SelectNodes("//td").ToList() : null;
                    if (tdList != null && tdList.Count() > 1)
                    {
                        B_SizeBallRanking sizing = new B_SizeBallRanking();
                        sizing.Id = Guid.NewGuid().ToString();
                        sizing.Sort = int.Parse(tdList[0].InnerText.Trim().ToString());
                        sizing.SizeBallType = sizeBallType;
                        sizing.TeamId = TeamList.Where(o => o.Name == tdList[1].InnerText.Trim().ToString()).Count() > 0 ? TeamList.Where(o => o.Name == tdList[1].InnerText.Trim().ToString()).FirstOrDefault().Id : "";
                        sizing.EventId = eventId;
                        sizing.Season = season;
                        sizing.Sum = int.Parse(tdList[2].InnerText.Trim().ToString());
                        sizing.LargeNum = int.Parse(tdList[3].InnerText.Trim().ToString());
                        sizing.GoNum = int.Parse(tdList[4].InnerText.Trim().ToString());
                        sizing.SmallNum = int.Parse(tdList[5].InnerText.Trim().ToString());
                        sizing.LargePercentage = tdList[6].InnerText.Trim().ToString();
                        sizing.GoPercentage = tdList[7].InnerText.Trim().ToString();
                        sizing.SmallPercentage = tdList[8].InnerText.Trim().ToString();
                        SizeBallRankingList.Add(sizing);
                    }
                }
            });
        }
    }
}
