﻿using System;
using System.Collections.Generic;
using FCS.Common;
using FCS.Crawler.Tools;
using FCS.Interface;
using FCS.Models;
using HtmlAgilityPack;
using Newtonsoft.Json;
using Quartz;
using System.Data;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Data.SqlClient;
using System.Threading.Tasks;
using System.Threading;
using System.Diagnostics;

namespace FCS.Crawler.ZCMatchRankingList
{
    /// <summary>
    /// 积分排行榜
    /// </summary>
    public class RankingListJob : CommonJob, IJob
    {

        private static List<F_Team> TeamList = new List<F_Team>();
        private static List<DataItemDetail> DateItem = new List<DataItemDetail>();
        private static List<F_IntegralRanking> RankingList = new List<F_IntegralRanking>();
        public RankingListJob()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
        }

        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
            GetAll();
        }

        #region 更新数据
        /// <summary>
        /// 获取所有数据
        /// </summary>
        public void GetAll()
        {
            GetSqlString(GetAllEventUrl, "");
        }

        /// <summary>
        /// 更新数据
        /// </summary>
        /// <param name="Year"></param>
        public void Click(string Year = "")
        {
            if (Year == "")
            {
                Year = DateTime.Now.Year.ToString();
            }
            var sql = string.Format(GetNewEventUrl, Year);
            GetSqlString(sql, Year);
        }



        #endregion

        /// <summary>
        /// 解析数据
        /// </summary>
        /// <param name="sqlstring"></param>
        private void GetSqlString(string sqlstring, string Year)
        {
            ThreadPool.SetMinThreads(10, 10);
            ThreadPool.SetMaxThreads(500, 500);
            TeamList = services.GetTeamList();
            var ds = SqlHelper.ExecuteDataset(CommandType.Text, sqlstring);
            if (ds != null && ds.Tables.Count > 0)
            {
                //获取某赛事赛季的html  ds.Tables[0].Rows.Count
                for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
                {
                    var url = ds.Tables[0].Rows[i]["Remark"].ToString();
                    var eventId = ds.Tables[0].Rows[i]["EventId"].ToString();
                    var season = ds.Tables[0].Rows[i]["Season"].ToString();
                    Task.Run(() =>
                    {
                        GetRanking(url, eventId, season);
                    });
                }
                int maxWorkerThreads, workerThreads;
                int maxportThreads, portThreads;
                while (true)
                {
                    /*
                     GetAvailableThreads()：检索由 GetMaxThreads 返回的线程池线程的最大数目和当前活动数目之间的差值。
                     而GetMaxThreads 检索可以同时处于活动状态的线程池请求的数目。
                     通过最大数目减可用数目就可以得到当前活动线程的数目，如果为零，那就说明没有活动线程，说明所有线程运行完毕。
                     */
                    ThreadPool.GetMaxThreads(out maxWorkerThreads, out maxportThreads);
                    ThreadPool.GetAvailableThreads(out workerThreads, out portThreads);
                    Thread.Sleep(1000);
                    Trace.WriteLine("正在执行任务的线程数" + (maxWorkerThreads - workerThreads));
                    if (maxWorkerThreads - workerThreads == 0)
                    {
                        Console.WriteLine("Thread Finished!");
                        break;
                    }

                }
                try
                {
                    if (Year != "")
                    {
                        services.Delete<F_MakeBallList>(" and Season like '" + Year + "%'");
                    }
                    services.SqlBulkCopyAdd<F_IntegralRanking>(RankingList);

                    Trace.WriteLine("F_IntegralRanking更新完毕");
                }
                catch (Exception ex)
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// 读取积分榜信息
        /// </summary>
        /// <param name="url"></param>
        public bool GetRanking(string url, string eventId, string season)
        {
            if (url.Contains("wwaattssuunn"))
            {
                return false;
            }
            HtmlDocument doc = CommonHelper.GetHtml(url, new Dictionary<string, string>(), "", "", 90000, 10);
            //获取积分榜列表
            var TongJiListData = doc.DocumentNode.SelectNodes("//*[@class='tabs1_main_ul']");
            if (TongJiListData == null || TongJiListData.Count == 0) return false;
            DateItem = services.GetDataItem(DataItemDetailEnum.FootIntegralRankingType);
            //总榜
            var RankingAll = TongJiListData[0];
            Rankinga(RankingAll, eventId, season, FootIntegralRankingEnum.AllRanking);
            //主场
            var RankingMain = TongJiListData[1];
            Rankinga(RankingMain, eventId, season, FootIntegralRankingEnum.MainRanking);
            //客场
            if (TongJiListData.Count > 2)
            {
                var RankingPassenger = TongJiListData[2];
                Rankinga(RankingPassenger, eventId, season, FootIntegralRankingEnum.PassengerRanking);
            }

            return true;
        }

        public void Rankinga(HtmlNode RankingAll, string eventId, string season, FootIntegralRankingEnum rankingEnum)
        {
            Task.Run(() =>
            {
                HtmlDocument doc1 = new HtmlDocument();
                doc1.LoadHtml(RankingAll.InnerHtml);
                var allmatchList = doc1.DocumentNode.SelectNodes("//table/tr");
                if (allmatchList == null || allmatchList.Count == 0) return;
                foreach (var allmatchItem in allmatchList)
                {
                    try
                    {
                        HtmlDocument doc11 = new HtmlDocument();
                        doc11.LoadHtml(allmatchItem.InnerHtml);
                        var tdList = doc11.DocumentNode.SelectNodes("//td");
                        if (tdList == null || tdList.Count == 0)
                        {
                            continue;
                        }
                        F_IntegralRanking f_IntegralRanking = new F_IntegralRanking();
                        f_IntegralRanking.Id = Guid.NewGuid().ToString();
                        f_IntegralRanking.EventId = eventId;
                        f_IntegralRanking.Season = season;
                        f_IntegralRanking.IntegralRankingType = DateItem.Where(o => o.ItemValue == ((int)rankingEnum).ToString()).Count() > 0 ? DateItem.Where(o => o.ItemValue == ((int)rankingEnum).ToString()).FirstOrDefault().Id : "";
                        f_IntegralRanking.Sort = tdList[0].InnerText.Trim() == null ? 0 : int.Parse(tdList[0].InnerText.Trim().ToString());
                        var TeamName = tdList[1].InnerText.Trim().ToString();
                        var Team = TeamList.Where(o => o.Name == TeamName);
                        f_IntegralRanking.TeamId = Team.Count() > 0 ? Team.FirstOrDefault().Id : TeamName;
                        f_IntegralRanking.Game = tdList[2].InnerText.Trim().ToString();
                        f_IntegralRanking.Win = int.Parse(tdList[3].InnerText.Trim().ToString());
                        f_IntegralRanking.Flat = int.Parse(tdList[4].InnerText.Trim().ToString());
                        f_IntegralRanking.Lose = int.Parse(tdList[5].InnerText.Trim().ToString());
                        f_IntegralRanking.Score = int.Parse(tdList[6].InnerText.Trim().ToString());
                        f_IntegralRanking.Loss = int.Parse(tdList[7].InnerText.Trim().ToString());
                        f_IntegralRanking.NetScore = int.Parse(tdList[8].InnerText.Trim().ToString());
                        f_IntegralRanking.ScoreAvg = double.Parse(tdList[9].InnerText.Trim().ToString());
                        f_IntegralRanking.LossAvg = double.Parse(tdList[10].InnerText.Trim().ToString());
                        f_IntegralRanking.WinRate = double.Parse(tdList[11].InnerText.Trim().Replace("%", "").ToString()) / 100;
                        f_IntegralRanking.FlatRate = double.Parse(tdList[12].InnerText.Trim().Replace("%", "").ToString()) / 100;
                        f_IntegralRanking.LoseRate = double.Parse(tdList[13].InnerText.Trim().Replace("%", "").ToString()) / 100;
                        f_IntegralRanking.Integral = int.Parse(tdList[14].InnerText.Trim().ToString());
                        f_IntegralRanking.CreateDateTime = DateTime.Now;
                        RankingList.Add(f_IntegralRanking);
                    }
                    catch (Exception ex)
                    {
                        throw;
                    }
                }
            });
        }



        #region SQL语句
        /// <summary>
        /// 获取所有的赛事
        /// </summary>
        private static string GetAllEventUrl = @"select distinct EventId,Season,Remark  from F_Grouping where Remark is not null";
        /// <summary>
        /// 获取最新两年的数据赛事
        /// </summary>
        private static string GetNewEventUrl = @"select distinct EventId,Season,Remark  from F_Grouping where Remark is not null and Season like '%{0}%'";

        #endregion
    }
}
